@@ -326,7 +326,7 @@ function(txt)
 {
    txt = paste(as.character(txt), collapse = "\n")
    enc = mapEncoding(Encoding(txt))
-   .Call("R_jsonPrettyPrint", txt, enc, package = "RJSONIO")
+   .Call("R_jsonPrettyPrint", txt, enc, PACKAGE = "RJSONIO")
 }
 
 
