@@ -1,86 +1,141 @@
 /*
     This is the implementation of the C interface to libjson
-    This file may be included in any C++ application, but it will 
-    be completely ignored if JSON_LIBRARY isn't defined.  The 
+    This file may be included in any C++ application, but it will
+    be completely ignored if JSON_LIBRARY isn't defined.  The
     only reason JSON_LIBRARY should be defined is when compiling libjson
     as a library
 */
-#include "../libjson.h"
+#include "libjson/libjson.h"
 #ifdef JSON_LIBRARY
 
     #include "JSONNode.h"
     #include "JSONWorker.h"
+    #include "JSONValidator.h"
+    #include "JSONStream.h"
+	#include "JSONGlobals.h"
     #include <stdexcept>  //some methods throw exceptions
     #ifdef JSON_MEMORY_MANAGE
-	   auto_expand StringHandler; 
-	   auto_expand_node NodeHandler;
-	   #define MANAGER_INSERT(x) NodeHandler.insert(x)
+	   #define MANAGER_INSERT(x) json_global(NODE_HANDLER).insert(x)
+	   #define MANAGER_STREAM_INSERT(x) json_global(STREAM_HANDLER).insert(x)
     #else
 	   #define MANAGER_INSERT(x) x
+	   #define MANAGER_STREAM_INSERT(x) x
     #endif
 
-    static const json_char * EMPTY_CSTRING = JSON_TEXT("");
+    static const json_char * EMPTY_CSTRING(JSON_TEXT(""));
 
-    inline json_char * toCString(const json_string & str){
-	   const size_t len((str.length() + 1) * sizeof(json_char));
+	#ifdef JSON_MEMORY_POOL
+		#include "JSONMemoryPool.h"
+		extern memory_pool<NODEPOOL> json_node_mempool;
+	#endif
+
+    inline json_char * toCString(const json_string & str) json_nothrow {
+	   const size_t len = (str.length() + 1) * sizeof(json_char);
 	   #ifdef JSON_MEMORY_MANAGE
-		  return (json_char *)StringHandler.insert(memcpy(json_malloc<json_char>(len), str.c_str(), len));
+		  return (json_char *)json_global(STRING_HANDLER).insert(std::memcpy(json_malloc<json_char>(len), str.c_str(), len));
 	   #else
-		  return (json_char *)memcpy(json_malloc<json_char>(len), str.c_str(), len);
+		  return (json_char *)std::memcpy(json_malloc<json_char>(len), str.c_str(), len);
 	   #endif
     }
-    
+	
+	inline json_char * alreadyCString(json_char * str) json_nothrow {
+		#ifdef JSON_MEMORY_MANAGE
+		   return (json_char *)json_global(STRING_HANDLER).insert(str);
+	    #else
+			return str;
+		#endif
+	}
+
     /*
 	   stuff that's in namespace libjson
     */
     void json_free(void * str){
 	   JSON_ASSERT_SAFE(str, JSON_TEXT("freeing null ptr"), return;);
 	   #ifdef JSON_MEMORY_MANAGE
-		  StringHandler.remove(str);
+		  json_global(STRING_HANDLER).remove(str);
 	   #endif
 	   libjson_free<void>(str);
     }
-    
+
     void json_delete(JSONNODE * node){
 	   JSON_ASSERT_SAFE(node, JSON_TEXT("deleting null ptr"), return;);
 	   #ifdef JSON_MEMORY_MANAGE
-		  NodeHandler.remove(node);
+		  json_global(NODE_HANDLER).remove(node);
 	   #endif
-	   JSONNode::deleteJSONNode((JSONNode *)node); 
+	   JSONNode::deleteJSONNode((JSONNode *)node);
     }
 
     #ifdef JSON_MEMORY_MANAGE
 	   void json_free_all(void){
-		  StringHandler.clear();
+		  json_global(STRING_HANDLER).clear();
 	   }
-	   
+
 	   void json_delete_all(void){
-		  NodeHandler.clear();
+		  json_global(NODE_HANDLER).clear();
+	   }
+    #endif
+
+    #ifdef JSON_READ_PRIORITY
+	   JSONNODE * json_parse(json_const json_char * json){
+		  JSON_ASSERT_SAFE(json, JSON_TEXT("null ptr to json_parse"), return 0;);
+		  json_try {
+			 //use this constructor to simply copy reference instead of copying the temp
+			 return MANAGER_INSERT(JSONNode::newJSONNode_Shallow(JSONWorker::parse(TOCONST_CSTR(json))));
+		  } json_catch (const std::invalid_argument&, (void)0; )
+		  #ifndef JSON_NO_EXCEPTIONS
+			 return 0;
+		  #endif
+	   }
+
+	   JSONNODE * json_parse_unformatted(json_const json_char * json){
+		  JSON_ASSERT_SAFE(json, JSON_TEXT("null ptr to json_parse"), return 0;);
+		  json_try {
+			 //use this constructor to simply copy reference instead of copying the temp
+			 return MANAGER_INSERT(JSONNode::newJSONNode_Shallow(JSONWorker::parse_unformatted(TOCONST_CSTR(json))));
+		  } json_catch(const std::invalid_argument&, (void)0; )
+		  #ifndef JSON_NO_EXCEPTIONS
+			 return 0;
+		  #endif
 	   }
     #endif
-    
-    JSONNODE * json_parse(json_const json_char * json){
-	   JSON_ASSERT_SAFE(json, JSON_TEXT("null ptr to json_parse"), return 0;);
-	   try {
-		  //use this constructor to simply copy reference instead of copying the temp
-		  return MANAGER_INSERT(JSONNode::newJSONNode_Shallow(JSONWorker::parse(TOCONST_CSTR(json))));
-	   } catch (std::invalid_argument){}
-	   return 0;
-    }
 
     json_char * json_strip_white_space(json_const json_char * json){
 	   JSON_ASSERT_SAFE(json, JSON_TEXT("null ptr to json_strip_white_space"), return 0;);
-	   return toCString(JSONWorker::RemoveWhiteSpaceAndComments(TOCONST_CSTR(json)));
+	   return alreadyCString(JSONWorker::RemoveWhiteSpaceAndCommentsC(TOCONST_CSTR(json), false));
     }
 
     #ifdef JSON_VALIDATE
-	   JSONNODE * json_validate(json_const json_char * json){
-		  JSON_ASSERT_SAFE(json, JSON_TEXT("null ptr to json_validate"), return 0;);
-		  try {
-			 //use this constructor to simply copy reference instead of copying the temp
-			 return MANAGER_INSERT(JSONNode::newJSONNode_Shallow(JSONWorker::validate(TOCONST_CSTR(json))));
-		  } catch (std::invalid_argument){}
-		  return 0;
+	   #ifdef JSON_DEPRECATED_FUNCTIONS
+		  JSONNODE * json_validate(json_const json_char * json){
+			 JSON_ASSERT_SAFE(json, JSON_TEXT("null ptr to json_validate"), return 0;);
+			 if (json_is_valid(json)){
+				return json_parse(json);
+			 }
+			 return 0;
+		  }
+	   #endif
+	   json_bool_t json_is_valid(json_const json_char * json){
+		  JSON_ASSERT_SAFE(json, JSON_TEXT("null ptr to json_is_valid"), return (json_bool_t)false;);
+		  #ifdef JSON_SECURITY_MAX_STRING_LENGTH
+			 if (json_unlikely(json_strlen(json) > JSON_SECURITY_MAX_STRING_LENGTH)){
+				JSON_FAIL(JSON_TEXT("Exceeding JSON_SECURITY_MAX_STRING_LENGTH"));
+				return false;
+			 }
+		  #endif
+		  json_auto<json_char> s;
+		  s.set(JSONWorker::RemoveWhiteSpaceAndCommentsC(json, false));
+		  return (json_bool_t)JSONValidator::isValidRoot(s.ptr);
+	   }
+
+	   json_bool_t json_is_valid_unformatted(json_const json_char * json){
+		  JSON_ASSERT_SAFE(json, JSON_TEXT("null ptr to json_is_valid_unformatted"), return (json_bool_t)true;);
+		  #ifdef JSON_SECURITY_MAX_STRING_LENGTH
+			 if (json_unlikely(json_strlen(json) > JSON_SECURITY_MAX_STRING_LENGTH)){
+				JSON_FAIL(JSON_TEXT("Exceeding JSON_SECURITY_MAX_STRING_LENGTH"));
+				return false;
+			 }
+		  #endif
+		  return (json_bool_t)JSONValidator::isValidRoot(json);
 	   }
     #endif
 
@@ -115,12 +170,12 @@
 
 	   void json_lock(JSONNODE * node, int threadid){
 		  JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_lock"), return;);
-		  ((JSONNode*)node) -> lock(threadid);   
+		  ((JSONNode*)node) -> lock(threadid);
 	   }
 
 	   void json_unlock(JSONNODE * node, int threadid){
 		  JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_unlock"), return;);
-		  ((JSONNode*)node) -> unlock(threadid);   
+		  ((JSONNode*)node) -> unlock(threadid);
 	   }
     #endif
 
@@ -130,6 +185,29 @@
 	   }
     #endif
 
+    #ifdef JSON_STREAM
+	   void json_stream_push(JSONSTREAM * stream, json_const json_char * addendum){
+		  (*((JSONStream*)stream)) << addendum;
+	   }
+
+	   void json_delete_stream(JSONSTREAM * stream){
+		  JSON_ASSERT_SAFE(stream, JSON_TEXT("deleting null ptr"), return;);
+		  #ifdef JSON_MEMORY_MANAGE
+			 json_global(STREAM_HANDLER).remove(stream);
+		  #endif
+		  JSONStream::deleteJSONStream((JSONStream *)stream);
+	   }
+
+	   JSONSTREAM * json_new_stream(json_stream_callback_t callback, json_stream_e_callback_t e_callback, void * identifier){
+		  return MANAGER_STREAM_INSERT(JSONStream::newJSONStream(callback, e_callback, identifier));
+	   }
+
+		void json_stream_reset(JSONSTREAM * stream){
+			JSON_ASSERT_SAFE(stream, JSON_TEXT("resetting null ptr"), return;);
+			((JSONStream*)stream) -> reset();
+		}
+    #endif
+
 
     /*
 	stuff that's in class JSONNode
@@ -138,56 +216,68 @@
     JSONNODE * json_new_a(json_const json_char * name, json_const json_char * value){
 	   if (!name) name = EMPTY_CSTRING;
 	   JSON_ASSERT_SAFE(value, JSON_TEXT("null value to json_new_a"), value = EMPTY_CSTRING;);
-	   #ifdef JSON_MEMORY_CALLBACKS
+		#ifdef JSON_MEMORY_POOL
+			return MANAGER_INSERT(new((JSONNode*)json_node_mempool.allocate()) JSONNode(TOCONST_CSTR(name), json_string(TOCONST_CSTR(value))));
+		#elif defined(JSON_MEMORY_CALLBACKS)
 		  return MANAGER_INSERT(new(json_malloc<JSONNode>(1)) JSONNode(TOCONST_CSTR(name), json_string(TOCONST_CSTR(value))));
 	   #else
 		  return MANAGER_INSERT(new JSONNode(TOCONST_CSTR(name), json_string(TOCONST_CSTR(value))));
-	   #endif 
+	   #endif
     }
 
-    JSONNODE * json_new_i(json_const json_char * name, long value){
-	   if (!name) name = EMPTY_CSTRING;	   
-	   #ifdef JSON_MEMORY_CALLBACKS
+    JSONNODE * json_new_i(json_const json_char * name, json_int_t value){
+	   if (!name) name = EMPTY_CSTRING;
+		#ifdef JSON_MEMORY_POOL
+			return MANAGER_INSERT(new((JSONNode*)json_node_mempool.allocate()) JSONNode(TOCONST_CSTR(name), value));
+		#elif defined(JSON_MEMORY_CALLBACKS)
 		  return MANAGER_INSERT(new(json_malloc<JSONNode>(1)) JSONNode(TOCONST_CSTR(name), value));
 	   #else
 		  return MANAGER_INSERT(new JSONNode(TOCONST_CSTR(name), value));
-	   #endif 
+	   #endif
     }
 
     JSONNODE * json_new_f(json_const json_char * name, json_number value){
-	   if (!name) name = EMPTY_CSTRING;	   
-	   #ifdef JSON_MEMORY_CALLBACKS
+	   if (!name) name = EMPTY_CSTRING;
+		#ifdef JSON_MEMORY_POOL
+			return MANAGER_INSERT(new((JSONNode*)json_node_mempool.allocate()) JSONNode(TOCONST_CSTR(name), value));
+		#elif defined(JSON_MEMORY_CALLBACKS)
 		  return MANAGER_INSERT(new(json_malloc<JSONNode>(1)) JSONNode(TOCONST_CSTR(name), value));
 	   #else
 		  return MANAGER_INSERT(new JSONNode(TOCONST_CSTR(name), value));
-	   #endif 
+	   #endif
     }
 
     JSONNODE * json_new_b(json_const json_char * name, json_bool_t value){
-	   if (!name) name = EMPTY_CSTRING;	   
-	   #ifdef JSON_MEMORY_CALLBACKS
-		  return MANAGER_INSERT(new(json_malloc<JSONNode>(1)) JSONNode(TOCONST_CSTR(name), (bool)value));
+	   if (!name) name = EMPTY_CSTRING;
+		#ifdef JSON_MEMORY_POOL
+			return MANAGER_INSERT(new((JSONNode*)json_node_mempool.allocate()) JSONNode(TOCONST_CSTR(name), static_cast<bool>(value)));
+		#elif defined(JSON_MEMORY_CALLBACKS)
+		  return MANAGER_INSERT(new(json_malloc<JSONNode>(1)) JSONNode(TOCONST_CSTR(name), static_cast<bool>(value)));
 	   #else
-		  return MANAGER_INSERT(new JSONNode(TOCONST_CSTR(name), (bool)value));
-	   #endif 
+		  return MANAGER_INSERT(new JSONNode(TOCONST_CSTR(name), static_cast<bool>(value)));
+	   #endif
     }
 
     JSONNODE * json_new(char type){
-	   #ifdef JSON_MEMORY_CALLBACKS
+		#ifdef JSON_MEMORY_POOL
+			return MANAGER_INSERT(new((JSONNode*)json_node_mempool.allocate()) JSONNode(type));
+	    #elif defined(JSON_MEMORY_CALLBACKS)
 		  return MANAGER_INSERT(new(json_malloc<JSONNode>(1)) JSONNode(type));
 	   #else
 		  return MANAGER_INSERT(new JSONNode(type));
-	   #endif 
+	   #endif
     }
 
-    JSONNODE * json_copy(json_const JSONNODE * orig){
-	   JSON_ASSERT_SAFE(orig, JSON_TEXT("null orig to json_copy"), return 0;);
-	   #ifdef JSON_MEMORY_CALLBACKS
-		  return MANAGER_INSERT(new(json_malloc<JSONNode>(1)) JSONNode(*((JSONNode*)orig)));
-	   #else
-		  return MANAGER_INSERT(new JSONNode(*((JSONNode*)orig)));
-	   #endif 
-    }
+	JSONNODE * json_copy(json_const JSONNODE * orig){
+		JSON_ASSERT_SAFE(orig, JSON_TEXT("null orig to json_copy"), return 0;);
+		#ifdef JSON_MEMORY_POOL
+			return MANAGER_INSERT(new((JSONNode*)json_node_mempool.allocate()) JSONNode(*((JSONNode*)orig)));
+		#elif defined(JSON_MEMORY_CALLBACKS)
+			return MANAGER_INSERT(new(json_malloc<JSONNode>(1)) JSONNode(*((JSONNode*)orig)));
+		#else
+			return MANAGER_INSERT(new JSONNode(*((JSONNode*)orig)));
+		#endif
+	}
 
     JSONNODE * json_duplicate(json_const JSONNODE * orig){
 	   JSON_ASSERT_SAFE(orig, JSON_TEXT("null orig to json_duplicate"), return 0;);
@@ -201,7 +291,7 @@
 	   *((JSONNode*)node) = json_string(TOCONST_CSTR(value));
     }
 
-    void json_set_i(JSONNODE * node, long value){
+    void json_set_i(JSONNODE * node, json_int_t value){
 	   JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_set_i"), return;);
 	   *((JSONNode*)node) = value;
     }
@@ -213,7 +303,7 @@
 
     void json_set_b(JSONNODE * node, json_bool_t value){
 	   JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_set_b"), return;);
-	   *((JSONNode*)node) = (bool)value;
+	   *((JSONNode*)node) = static_cast<bool>(value);
     }
 
     void json_set_n(JSONNODE * node, json_const JSONNODE * orig){
@@ -254,51 +344,80 @@
     json_char * json_as_string(json_const JSONNODE * node){
 	   JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_as_string"), return toCString(EMPTY_CSTRING););
 	   return toCString(((JSONNode*)node) -> as_string());
+	   //return toCString(static_cast<json_string>(*((JSONNode*)node)));
     }
 
-    long json_as_int(json_const JSONNODE * node){
+    json_int_t json_as_int(json_const JSONNODE * node){
 	   JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_as_int"), return 0;);
 	   return ((JSONNode*)node) -> as_int();
+	   //return static_cast<json_int_t>(*((JSONNode*)node));
     }
 
     json_number json_as_float(json_const JSONNODE * node){
 	   JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_as_float"), return 0.0f;);
 	   return ((JSONNode*)node) -> as_float();
+	   //return static_cast<json_number>(*((JSONNode*)node));
     }
 
     json_bool_t json_as_bool(json_const JSONNODE * node){
 	   JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_as_bool"), return false;);
-	   return (json_bool_t)(((JSONNode*)node) -> as_bool());
+	   return ((JSONNode*)node) -> as_bool();
+	   //return (json_bool_t)static_cast<bool>(*((JSONNode*)node));
     }
 
-    JSONNODE * json_as_node(json_const JSONNODE * node){
-	   JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_as_node"), return 0;);
-	   return MANAGER_INSERT(JSONNode::newJSONNode_Shallow(((JSONNode*)node) -> as_node()));
-    }
+	#ifdef JSON_CASTABLE
+		JSONNODE * json_as_node(json_const JSONNODE * node){
+		   JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_as_node"), return 0;);
+		   return MANAGER_INSERT(JSONNode::newJSONNode_Shallow(((JSONNode*)node) -> as_node()));
+		}
 
-    JSONNODE * json_as_array(json_const JSONNODE * node){
-	   JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_as_array"), return 0;);
-	   return MANAGER_INSERT(JSONNode::newJSONNode_Shallow(((JSONNode*)node) -> as_array()));
-    }
+		JSONNODE * json_as_array(json_const JSONNODE * node){
+		   JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_as_array"), return 0;);
+		   return MANAGER_INSERT(JSONNode::newJSONNode_Shallow(((JSONNode*)node) -> as_array()));
+		}
+	#endif
 
-    #ifdef JSON_BINARY
-	   void * json_as_binary(json_const JSONNODE * node, unsigned long * size){
-		  JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_as_binary"), if (size){*size = 0;} return 0;);
-		  const std::string result = ((JSONNode*)node) -> as_binary();
+    #if defined(JSON_BINARY) || defined(JSON_EXPOSE_BASE64)
+	   static void * returnDecode64(const std::string & result, unsigned long * size) json_nothrow json_cold;
+	   static void * returnDecode64(const std::string & result, unsigned long * size) json_nothrow {
 		  const size_t len = result.length();
-		  if (size) *size = (json_index_t)len;
+		  if (json_likely(size)) *size = (json_index_t)len;
 		  #ifdef JSON_SAFE
-			 if (result.empty()) return 0;
+			 if (json_unlikely(result.empty())) return 0;
 		  #endif
 		  #ifdef JSON_MEMORY_MANAGE
-			 return StringHandler.insert(memcpy(json_malloc<char>(len), result.data(), len));
+			 return json_global(STRING_HANDLER).insert(std::memcpy(json_malloc<char>(len), result.data(), len));
 		  #else
-			 return memcpy(json_malloc<char>(len), result.data(), len);
+			 return std::memcpy(json_malloc<char>(len), result.data(), len);
 		  #endif
 	   }
     #endif
 
-    #ifdef JSON_WRITER
+    #ifdef JSON_BINARY
+	   void * json_as_binary(json_const JSONNODE * node, unsigned long * size){
+		  JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_as_binary"), if (size){*size = 0;} return 0;);
+		  return returnDecode64(((JSONNode*)node) -> as_binary(), size);
+
+	   }
+    #endif
+
+    #ifdef JSON_EXPOSE_BASE64
+	   #include "JSON_Base64.h"
+	   json_char * json_encode64(json_const void * binary, json_index_t bytes){
+		  const json_string result(JSONBase64::json_encode64((const unsigned char *)binary, (size_t)bytes));
+		  #ifdef JSON_MEMORY_MANAGE
+			 return (json_char*)json_global(STRING_HANDLER).insert((json_char*)std::memcpy(json_malloc<json_char>(result.length() + 1), result.c_str(), (result.length() + 1) * sizeof(json_char)));
+		  #else
+			 return (json_char*)std::memcpy(json_malloc<json_char>(result.length() + 1), result.c_str(), (result.length() + 1) * sizeof(json_char));
+		  #endif
+	   }
+
+	   void * json_decode64(const json_char * text, unsigned long * size){
+		  return returnDecode64(JSONBase64::json_decode64(text), size);
+	   }
+    #endif
+
+    #ifdef JSON_WRITE_PRIORITY
 	   json_char * json_write(json_const JSONNODE * node){
 		  JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_write"), return toCString(EMPTY_CSTRING););
 		  return toCString(((JSONNode*)node) -> write());
@@ -347,7 +466,7 @@
 	   ((JSONNode*)node) -> merge(*(JSONNode*)node2);
     }
 
-    #ifndef JSON_PREPARSE
+    #if !defined(JSON_PREPARSE) && defined(JSON_READ_PRIORITY)
 	   void json_preparse(JSONNODE * node){
 		  JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_preparse"), return;);
 		  ((JSONNode*)node) -> preparse();
@@ -362,10 +481,12 @@
 	   }
     #endif
 
-    void json_cast(JSONNODE * node, char type){
-	   JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_cast"), return;);
-	   ((JSONNode*)node) -> cast(type);
-    }
+    #ifdef JSON_CASTABLE
+	   void json_cast(JSONNODE * node, char type){
+		  JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_cast"), return;);
+		  ((JSONNode*)node) -> cast(type);
+	   }
+    #endif
 
     //children access
     void json_reserve(JSONNODE * node, json_index_t siz){
@@ -375,19 +496,23 @@
 
     JSONNODE * json_at(JSONNODE * node, unsigned int pos){
 	   JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_at"), return 0;);
-	   try {
+	   json_try {
 		  return &((JSONNode*)node) -> at(pos);
-	   } catch (std::out_of_range){}
-	   return 0;
+	   } json_catch (const std::out_of_range&, (void)0; )
+	   #ifndef JSON_NO_EXCEPTIONS
+		  return 0;
+	   #endif
     }
 
     JSONNODE * json_get(JSONNODE * node, json_const json_char * name){
 	   JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_get"), return 0;);
 	   JSON_ASSERT_SAFE(name, JSON_TEXT("null node to json_get.  Did you mean to use json_at?"), return 0;);
-	   try {
+	   json_try {
 		  return &((JSONNode*)node) -> at(TOCONST_CSTR(name));
-	   } catch (std::out_of_range){}
-	   return 0;
+	   } json_catch (const std::out_of_range&, (void)0; )
+	   #ifndef JSON_NO_EXCEPTIONS
+		  return 0;
+	   #endif
     }
 
 
@@ -395,10 +520,12 @@
 	   JSONNODE * json_get_nocase(JSONNODE * node, json_const json_char * name){
 		  JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_at_nocase"), return 0;);
 		  JSON_ASSERT_SAFE(name, JSON_TEXT("null name to json_at_nocase"), return 0;);
-		  try {
+		  json_try {
 			 return &((JSONNode*)node) -> at_nocase(TOCONST_CSTR(name));
-		  } catch (std::out_of_range){}
-		  return 0;
+		  } json_catch (const std::out_of_range&, (void)0; )
+		  #ifndef JSON_NO_EXCEPTIONS
+			 return 0;
+		  #endif
 	   }
 
 	   JSONNODE * json_pop_back_nocase(JSONNODE * node, json_const json_char * name){
@@ -412,7 +539,7 @@
 	   JSON_ASSERT_SAFE(node, JSON_TEXT("null node to json_push_back"), return;);
 	   JSON_ASSERT_SAFE(node2, JSON_TEXT("null node2 to json_push_back"), return;);
 	   #ifdef JSON_MEMORY_MANAGE
-		  NodeHandler.remove(node2);
+		  json_global(NODE_HANDLER).remove(node2);
 	   #endif
 	   ((JSONNode*)node) -> push_back((JSONNode*)node2);
     }
@@ -427,7 +554,7 @@
 	   JSON_ASSERT_SAFE(name, JSON_TEXT("null name to json_pop_back.  Did you mean to use json_pop_back_at?"), return 0;);
 	   return MANAGER_INSERT(((JSONNode*)node) -> pop_back(TOCONST_CSTR(name)));
     }
-    
+
     #ifdef JSON_ITERATORS
 	   JSONNODE_ITERATOR json_find(JSONNODE * node, json_const json_char * name){
 		  return (JSONNODE_ITERATOR)(((JSONNode*)node) -> find(TOCONST_CSTR(name)));
@@ -449,7 +576,7 @@
 
 	   JSONNODE_ITERATOR json_insert(JSONNODE * node, JSONNODE_ITERATOR it, JSONNODE * node2){
 		  #ifdef JSON_MEMORY_MANAGE
-			 NodeHandler.remove(node2);
+			 json_global(NODE_HANDLER).remove(node2);
 		  #endif
 		  return (JSONNODE_ITERATOR)(((JSONNode*)node) -> insert((JSONNode**)it, (JSONNode*)node2));
 	   }
@@ -462,7 +589,7 @@
 	   JSONNODE_ITERATOR json_begin(JSONNODE * node){
 		  return (JSONNODE_ITERATOR)(((JSONNode*)node) -> begin());
 	   }
-	   
+
 	   JSONNODE_ITERATOR json_end(JSONNODE * node){
 		  return (JSONNODE_ITERATOR)(((JSONNode*)node) -> end());
 	   }
