@@ -4,7 +4,7 @@
 #include "JSONDebug.h"
 #if defined(JSON_BINARY) || defined(JSON_EXPOSE_BASE64)  //if this is not needed, don't waste space compiling it
 
-#include "../Dependencies/libbase64++/libbase64++.h"
+#include "libjson/Dependencies/libbase64++/libbase64++.h"
 
 class JSONBase64 {
 public:
